ZOOM_OUT = {
	'40.7590615, -73.969231': 'Manhattan, NY',
}

MAX_NODE_DIFFERENCE = 100
MAX_NODE_BACKTRACK = 50

MENU_BORDERS = [[15, 123], [30, 429]]

WATER_RED = list(range(148, 185))
WATER_GREEN = list(range(193, 215))
WATER_BLUE = list(range(253, 256))

ALLOWABLE_RED = list(range(218, 241))
ALLOWABLE_GREEN = list(range(0, 191))
ALLOWABLE_BLUE = list(range(65, 181))